/*****************************************************************************
******************************************************************************/

#include <string.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include "LPC29xx.h"                        /* LPC29xx definitions */
#include "type.h"
#include "target.h"
#include "irq.h"
#include <intrinsics.h> /* for interrupts iar */
#include "timer.h"
#include "drv_hd44780.h"
#include "suart.h"
extern volatile DWORD timer_counter;
extern void SUART_Init(void);

static void setled(int led, int state)
{
	switch(led) {
		case 3:
			if(state) {
				GPIO0_OR &= ~(1<<28);
			} else {
				GPIO0_OR |= (1<<28);
			}
			break;
		case 1:
			if(state) {
				GPIO3_OR &= ~(1<<0);
			} else {
				GPIO3_OR |= (1<<0);
			}
			break;
		case 2:
			if(state) {
				GPIO3_OR &= ~(1<<12);
			} else {
				GPIO3_OR |= (1<<12);
			}
			break;
	}
}

void Dly100us(void *arg)
{
	for(volatile int i=0; i<10*1000; ++i);
}

static void animation_delay(void)
{
	timer_counter = 0;
	while(timer_counter < 1);
}

static void animate_powerup(void)
{
	HD44780_StrShow(1,1,"________");
	animation_delay();
	HD44780_StrShow(1,1,"S_______");
	animation_delay();
	HD44780_StrShow(1,1,"_T______");
	animation_delay();
	HD44780_StrShow(1,1,"__A_____");
	animation_delay();
	HD44780_StrShow(1,1,"___R____");
	animation_delay();
	HD44780_StrShow(1,1,"____T___");
	animation_delay();
	HD44780_StrShow(1,1,"_____I__");
	animation_delay();
	HD44780_StrShow(1,1,"______N_");
	animation_delay();
	HD44780_StrShow(1,1,"_______G");
	animation_delay();
}
/**************** LIN ***********************/
typedef union 
{
  char byte[4];
  int  dword;
} lindata;

#define ID_SEND     0x47
#define ID_RECEIVE  0x85

/*
LIN check sum calculation
*/
char LinCheckSum(char checksum, char data)
{
  union
  {
    struct
    {
      char lo;
      char hi;
    };
    short cs;
  } cs;
  
  cs.cs = checksum + data;
  
  return cs.hi+cs.lo;
}

/**************** Master LIN ****************/
/*
Master LIN controller 0 Init
*/
void LIN0_Init(int baudrate)
{
  /*Pin Init*/
  SFSP0_8 = 6;
  SFSP0_9 = 6;
  /*13-bits synch brake lenght
    1-bit inter byte space lenght
    Hardware check sum and ID parity */
  LIN0_LCFG = 0x0B;
  /*Baud rate*/
  int tmp = (PERIPH_CLK+baudrate/2)/baudrate;
  LIN0_LFBRG = tmp>>4;
  LIN0_LFBRG |= (tmp&0xF)<<16;
  /*126 bits time out*/
  LIN0_LTO = 0x7E;
  /*LIN Master mode*/
  LIN0_LMODE = 0;
}

/*
Master Start Transfer
*/
void LinStartTransfer(char dir, char size, char id, char * data)
{
  LIN0_LID = ((dir?1:0)<<24)  |
             ((size&0x1F)<<16)|
             ((id&0x3F)<<0)   ;
  if(!dir && data)
  {/*data copy*/
     memcpy((char *)&LIN0_LDATA,data,size);
  }
 
  LIN0_LCMD = 1;
}

/**************** Slave  LIN ****************/
typedef enum 
{
  BREAK = 0,
  SYNCH,
  RECEIVE_ID,
  RECEIVE_DATA
} slvst;

/*
Slave State machine
*/
void SlaveLIN(void)
{
static slvst SlaveState = BREAK;
static char SlvCS = 0;
static char SlvDataSize = 0;

  char status = SUART_GetStatus();
  
  if(status & RXB_NE)
  {/**/
    char data = SUART_GetChar();
    switch(SlaveState)
    {
    case BREAK:
      if((0x00 == data)&& (status & FRM_ERR))
      {/**/
        SlaveState = SYNCH;
      }
      break;
    case SYNCH:
      if((0x55 == data)&&!(status & FRM_ERR))
      {
        SlaveState = RECEIVE_ID;
        break;
      }
    
      SlaveState = BREAK;
      break;
    case RECEIVE_ID:
      if(!(status & 0x1))
      {
        if((ID_SEND == data))
        {
          SlvDataSize = sizeof(lindata);
          SlaveState = RECEIVE_DATA;
          break;
        }
        else
        {
          if((ID_RECEIVE == data))
          {
            SlvDataSize = sizeof(lindata);
            char trcs = 0;
            lindata rnd;
            rnd.dword = rand();
            
            for(int i = 0; i < SlvDataSize ;i++)
            {
              SUART_PutChar(rnd.byte[i]);
              trcs = LinCheckSum(trcs,rnd.byte[i]);
            }
            
            SUART_PutChar(0xFF^trcs);
            SlaveState = RECEIVE_DATA;
            break;
          }
        }
      }
      
      SlaveState = BREAK;
      break;
    case RECEIVE_DATA:
      SlvCS = LinCheckSum(SlvCS,data);
      if(0 == SlvDataSize--)
      {
        SlaveState = BREAK;
        if( 0xFF == SlvCS)
        {
          GPIO3_OR ^= (1<<0);
        }
        
        SlvCS = 0;
      }
        
      break;
    }
  }
}

/*****************************************************************************
**   Main Function  main()
******************************************************************************/
int main (void)
{          
  volatile unsigned int dummy;

  HD44780_ERROR_CODE_DEF lcdinit_st = HD44780_OK;

  int LINdir=0;
  /* Enable MSCSS timer for ADC event trigger, match0 for ADC0 */
  init_timer(TIMER_TICKS);
  enable_timer();
  /* GPIO 3.0 and 3.12 are LED pins set to output */
  GPIO0_DR = (1<<28);
  GPIO3_DR = (1<<0)|(1<<12);
  SFSP0_28 = (1<<2);
  SFSP3_0 = (1<<2);
  SFSP3_12 = (1<<2);

  SFSP2_26 = (1<<2);
  SFSP2_27 = (1<<2);

  setled(1, 0);
  setled(2, 0);
  setled(3, 1);

  __enable_interrupt(); // iar
  /*Lin0 is maseter*/
  LIN0_Init(19200);
  /*SUART is LIN slave.
    Baud rate is defined in suart_cfg.h*/
  SUART_Init();
 
  /* HD44780 LCD driver init */
  lcdinit_st = HD44780_PowerUpInit();
  if(lcdinit_st != HD44780_OK) {
	  for(;;) {
		  setled(1,1);
		  setled(2,1);
		  setled(3,1);
	  }
  }

  animate_powerup();
  
  HD44780_StrShow(1,1,"*OLIMEX*");

  for(;;) {
  /*Master Start transfer*/  
  if(timer_counter)
  {
    if(LINdir)
    {/*Start Master to Slave transfer*/
      lindata rnd;
      rnd.dword = rand();
      LinStartTransfer(0, 4, ID_SEND& 0x3F, rnd.byte);
    }
    else
    {/*Start Slave to Master transfer*/
      LinStartTransfer(1, 4, ID_RECEIVE& 0x3F, NULL);
    }
 
    LINdir ^=0x1;
    timer_counter = 0;
  }
  /*Master check for received frame*/   
  if(LIN0_LSTAT & 0x1)
  {
    char checksum = 0;
    char * mdata = (char *)&LIN0_LDATA;
    for(int i = ((LIN0_LID>>16) &0x1F); i; i--)
    {
      checksum = LinCheckSum(checksum,*mdata);
      mdata++;
    }
     
    if( 0xFF == LinCheckSum(checksum,LIN0_LCS))
    {/*Master Received frame is ok*/
      GPIO3_OR ^= (1<<12);
    }
    /*This should clear MR bit*/
    LIN0_LDATA = 0;
  }
  
  /*Slave Lin state machine*/
  SlaveLIN();

  }
  //return 0;
}

/*****************************************************************************
**                            End Of File
*****************************************************************************/

