/*****************************************************************************
 *   adctest.c:  main C entry file for NXP LPC29xx Family Microprocessors
 *
 *   Copyright(C) 2007, NXP Semiconductor
 *   All rights reserved.
 *
 *   History
 *   2007.08.16  ver 1.00    Prelimnary version, first Release
 *
******************************************************************************/

#include <string.h>
#include <limits.h>
#include <stdio.h>
#include "LPC29xx.h"                        /* LPC29xx definitions */
#include "type.h"
#include "target.h"
#include "irq.h"
#include "adc.h"
#include "uart.h"
#include "mtimer.h"
#include <intrinsics.h> /* for interrupts iar */
#include "timer.h"
#include "uart.h"
#include "drv_hd44780.h"

extern volatile DWORD timer_counter;

extern volatile DWORD UART0Count;
extern volatile BYTE UART0Buffer[BUFSIZE];

extern volatile DWORD ADC2Value[ITERATION][ADC_NUM];
extern volatile DWORD ADC2ValueAverage[ADC_NUM];
extern volatile DWORD ADC2IntDone;
extern volatile DWORD ADC2Iteration;

static void setled(int led, int state)
{
	switch(led) {
		case 3:
			if(state) {
				GPIO0_OR &= ~(1<<28);
			} else {
				GPIO0_OR |= (1<<28);
			}
			break;
		case 1:
			if(state) {
				GPIO3_OR &= ~(1<<0);
			} else {
				GPIO3_OR |= (1<<0);
			}
			break;
		case 2:
			if(state) {
				GPIO3_OR &= ~(1<<12);
			} else {
				GPIO3_OR |= (1<<12);
			}
			break;
	}
}

void Dly100us(void *arg)
{
	for(volatile int i=0; i<10*1000; ++i);
}


int uart0_getchar_nonblocking(void)
{
	int c;

	if(UART0Count != 0)
	{
		U0IER = IER_THRE | IER_RLS;	/* Disable RBR */
		c = UART0Buffer[0];		/* ignore rest of input */
		UART0Count = 0;
		U0IER = IER_THRE | IER_RLS | IER_RBR;	/* Re-enable RBR */
	} else {
		c = -1;
	}
	return c;
}

void uart0_print(const char *s)
{
	char cr = '\r';
	
	while(*s) {
		if(*s == '\n')
			UART0Send((BYTE*)&cr, 1);
		UART0Send((BYTE*)s, 1);
		s++;
	}
}

static int trimmmer_test(void)
{
	int stat = 0;
	static char buf[128];
	int adc_val, adc_min=1, adc_max=1;
	int pos, n;
	
	while(uart0_getchar_nonblocking() < 0) {
		if ( ADC2IntDone )
		{
			adc_val = ADC2Value[0][0];
			ADC2IntDone = 0;
			ADC2_Start();

			// indicate by the LEDs
			setled(1, (adc_val >= 1*1024/4));
			setled(2, (adc_val >= 2*1024/4));
			setled(3, (adc_val >= 3*1024/4));
			
			sprintf(buf,"\rTRIM=%d        ", adc_val);
			uart0_print(buf);
			
			
			// indicate on the LCD				
			for(n=adc_val,pos=8; pos>=1; pos--) {
				char s[2] = {'X','\0'};
				int c = n%10;
				n /= 10;
				if(!n && pos==8) {
					s[0] = '0';
				} else if(n || c) {
					s[0] = c + '0';
				} else {
					s[0] = ' ';
				}
				HD44780_StrShow(pos,1,(Int8S*)s);
			}
			
			if(adc_min > adc_val)
				adc_min = adc_val;
			if(adc_max < adc_val)
				adc_max = adc_val;
		}
	}
	uart0_print("\r                      \n");
	if(adc_max != 1023) {
		sprintf(buf,"ERROR: Expected adc_max=1024, got %d\n",adc_max);
		stat = -1;
	}
	if(adc_min != 0) {
		sprintf(buf,"ERROR: Expected adc_min=0, got %d\n",adc_min);
		stat = -1;
	}
	return stat;
}

/*-------------------------------------------------------------------------*/

static void animation_delay(void)
{
	timer_counter = 0;
	while(timer_counter < 1);
}

static void animate_powerup(void)
{
	HD44780_StrShow(1,1,"________");
	animation_delay();
	HD44780_StrShow(1,1,"S_______");
	animation_delay();
	HD44780_StrShow(1,1,"_T______");
	animation_delay();
	HD44780_StrShow(1,1,"__A_____");
	animation_delay();
	HD44780_StrShow(1,1,"___R____");
	animation_delay();
	HD44780_StrShow(1,1,"____T___");
	animation_delay();
	HD44780_StrShow(1,1,"_____I__");
	animation_delay();
	HD44780_StrShow(1,1,"______N_");
	animation_delay();
	HD44780_StrShow(1,1,"_______G");
	animation_delay();
}


/*****************************************************************************
**   Main Function  main()
******************************************************************************/
int main (void)
{
  HD44780_ERROR_CODE_DEF lcdinit_st = HD44780_OK;

  UART0Init(115200);

  /* Initialize ADC  */
  ADC2_Init();
  ADC2_Start();

  /* Enable MSCSS timer for ADC event trigger, match0 for ADC0 */
  init_mtimer0(MTIMER_TICKS);
  enable_mtimer0();
  init_timer(TIMER_TICKS);
  enable_timer();

  /* GPIO 3.0 and 3.13 are LED pins set to output */
  GPIO0_DR = (1<<28);
  GPIO3_DR = (1<<0)|(1<<12);
  SFSP0_28 = (1<<2);
  SFSP3_0 = (1<<2);
  SFSP3_13 = (1<<2);

  SFSP2_26 = (1<<2);
  SFSP2_27 = (1<<2);

  setled(1, 0);
  setled(2, 0);
  setled(3, 1);

 __enable_interrupt(); // iar

  /* HD44780 LCD driver init */
  lcdinit_st = HD44780_PowerUpInit();
  if(lcdinit_st != HD44780_OK) {
	  for(;;) {
		  setled(1,1);
		  setled(2,1);
		  setled(3,1);
	  }
  }

  ADC2Iteration = 0;

  uart0_print("\n\nWelcome to the LPC-P2919 Trimmer test.\n");

  animate_powerup();

  trimmmer_test();

  uart0_print("\n\nTest terminated, please restart!\n");
  HD44780_StrShow(1,1,"RESTART ");

  while(1);

  //return 0;
}

/*****************************************************************************
**                            End Of File
*****************************************************************************/

