/*****************************************************************************
 *   adctest.c:  main C entry file for NXP LPC29xx Family Microprocessors
 *
 *   Copyright(C) 2007, NXP Semiconductor
 *   All rights reserved.
 *
 *   History
 *   2007.08.16  ver 1.00    Prelimnary version, first Release
 *
******************************************************************************/

#include <string.h>
#include <limits.h>
#include <stdio.h>
#include "LPC29xx.h"                        /* LPC29xx definitions */
#include "type.h"
#include "target.h"
#include "irq.h"
#include "uart.h"
#include <intrinsics.h> /* for interrupts iar */
#include "timer.h"
#include "uart.h"
#include "drv_hd44780.h"

#include "testlib/stdest.h"

extern volatile DWORD timer_counter;

extern volatile DWORD UART0Count;
extern volatile BYTE UART0Buffer[BUFSIZE];

static void setled(int led, int state)
{
	switch(led) {
		case 3:
			if(state) {
				GPIO0_OR &= ~(1<<28);
			} else {
				GPIO0_OR |= (1<<28);
			}
			break;
		case 1:
			if(state) {
				GPIO3_OR &= ~(1<<0);
			} else {
				GPIO3_OR |= (1<<0);
			}
			break;
		case 2:
			if(state) {
				GPIO3_OR &= ~(1<<12);
			} else {
				GPIO3_OR |= (1<<12);
			}
			break;
	}
}

void Dly100us(void *arg)
{
	for(volatile int i=0; i<10*1000; ++i);
}


int uart0_getchar_nonblocking(void)
{
	int c;

	if(UART0Count != 0)
	{
		U0IER = IER_THRE | IER_RLS;	/* Disable RBR */
		c = UART0Buffer[0];		/* ignore rest of input */
		UART0Count = 0;
		U0IER = IER_THRE | IER_RLS | IER_RBR;	/* Re-enable RBR */
	} else {
		c = -1;
	}
	return c;
}

void uart0_print(const char *s)
{
	char cr = '\r';
	
	while(*s) {
		if(*s == '\n')
			UART0Send((BYTE*)&cr, 1);
		UART0Send((BYTE*)s, 1);
		s++;
	}
}

static void animation_delay(void)
{
	timer_counter = 0;
	while(timer_counter < 1);
}

static void animate_powerup(void)
{
	HD44780_StrShow(1,1,"________");
	animation_delay();
	HD44780_StrShow(1,1,"S_______");
	animation_delay();
	HD44780_StrShow(1,1,"_T______");
	animation_delay();
	HD44780_StrShow(1,1,"__A_____");
	animation_delay();
	HD44780_StrShow(1,1,"___R____");
	animation_delay();
	HD44780_StrShow(1,1,"____T___");
	animation_delay();
	HD44780_StrShow(1,1,"_____I__");
	animation_delay();
	HD44780_StrShow(1,1,"______N_");
	animation_delay();
	HD44780_StrShow(1,1,"_______G");
	animation_delay();
}


/*****************************************************************************
**   Main Function  main()
******************************************************************************/
int main (void)
{
	HD44780_ERROR_CODE_DEF lcd_st = HD44780_OK;
	int stat;
	
	UART0Init(115200);
	
	/* Enable timer*/
	init_timer(TIMER_TICKS);
	enable_timer();
	
	/* GPIO 3.0 and 3.13 are LED pins set to output */
	GPIO0_DR = (1<<28);
	GPIO3_DR = (1<<0)|(1<<12);
	SFSP0_28 = (1<<2);
	SFSP3_0 = (1<<2);
	SFSP3_13 = (1<<2);
	
	SFSP2_26 = (1<<2);
	SFSP2_27 = (1<<2);
	
	setled(1, 0);
	setled(2, 0);
	setled(3, 1);
	
	__enable_interrupt(); // iar
	
	/* HD44780 LCD driver init */
	lcd_st = HD44780_PowerUpInit();
	if(lcd_st != HD44780_OK) {
		for(;;) {
			setled(1,1);
			setled(2,1);
			setled(3,1);
		}
	}
	
	uart0_print("\n\nWelcome to the LPC-P2919 SD card slot test.\n");
	uart0_print("Please wait...\n\n");
	
	animate_powerup();
	lcd_st = HD44780_StrShow(1,1,"*OLIMEX*");
	if(lcd_st != HD44780_OK) {
		for(;;) {
			setled(1,1);
			setled(2,1);
			setled(3,1);
		}
	}

	// here the test is performed
	stat = sdtest();
	if(stat) {
		uart0_print("SD slot test FAILED!!\n");
	} else {
		uart0_print("SD slot test finished successfully!!\n");
	}

	uart0_print("\n\nTest terminated, please restart!\n");
	HD44780_StrShow(1,1,"RESTART ");

  //return 0;
}

/*****************************************************************************
**                            End Of File
*****************************************************************************/

