/****************************************************************************
 *
 * Project: -----------
 *
 * Copyright by Olimex Ltd. All rights reserved.
 *
 * File: stdest.c
 * Description: SD slot test
 * Developer: Dimitar Dimitrov ( dinuxbg@gmail.com )
 *
 * Last change: $Date: 2009-04-02 15:07:10 +0300 (Thu, 02 Apr 2009) $
 * Revision: $Revision: 29 $
 * Id: $Id: stdest.c 680 2009-04-02 12:07:10Z dimitar $
 *
 ****************************************************************************/

#include <assert.h>
#include <stddef.h>
#include <stdint.h>
#include <stdbool.h>
#include <stdio.h>

#include "stdest.h"

#include "../sbblib/sd_spi_mode.h"
#include "../sbblib/sd_ll_spi0.h"

#include <NXP/iolpc2919.h>

/**
 * @file
 * SD slot test
 *
 *
 */

int uart0_getchar_nonblocking(void);
void uart0_print(const char *s);


void sd_pwr(int on)
{
	SFSP2_9 = (0x01<<2)|(0x00<<0);
	if(!on)
		GPIO2_OR |= (0x1<<9);
	else
		GPIO2_OR &= ~(0x1<<9);
	GPIO2_DR |= (0x1<<9);
}


static void delay(volatile int i)
{
	while(i--);
}

int sdtest(void)
{
	static char buf[256];
	pDiskCtrlBlk_t info;
	bool disk_info_ready;
	
	
	uart0_print("Remove the SD/MMC card!\n");
	
	sd_pwr(0);
	uart0_print("Check that the SD LED is OFF (press key to continue)");
	while(uart0_getchar_nonblocking() < 0);
	
	sd_pwr(1);
	uart0_print("\n");
	uart0_print("Check that the SD LED is ON (press key to continue)");
	while(uart0_getchar_nonblocking() < 0);
	uart0_print("\n");

	uart0_print("Please insert an SD/MMC card...");
	
	do {
		if(SdPresent()) {
			SdDiskInit();

			info = SdGetDiskCtrlBkl();

			if((info->DiskStatus != DiskNotPresent)
					&& (info->DiskStatus != DiskCommandPass)
					&& (info->DiskStatus != DiskNotReady))
			{
				uart0_print("SD/MMC ERROR\n");
				return -1;
			}
			disk_info_ready = (info->DiskStatus == DiskCommandPass);
		} else {
			disk_info_ready = false;
		}
	} while(!disk_info_ready);
	
	delay(100000);
	uart0_print("\n");
	sprintf(buf, "Blocks num %d\n", info->BlockNumb);
	uart0_print(buf);
	sprintf(buf, "Block size %d\n", info->BlockSize);
	uart0_print(buf);
	if(info->BlockSize == 0) {
		uart0_print("ERROR: Invalid block size!\n");
		return -1;
	}
	sprintf(buf, "SD size %dM\n", ((info->BlockNumb / 1024) * info->BlockSize) /1024 );
	uart0_print(buf);
	sprintf(buf, "WriteProtect=%d\n", info->WriteProtect);
	uart0_print(buf);

	return 0;
}